﻿using KMeansProject;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace DiscoverClustersKMeans
{
    public partial class Form1 : Form
    {
        private BindingList<Point> _dataPointList;
        private BindingList<DPoint> _kmeansPointList;
        private Series _dataSeries;
        private Series _kMeansSeries;

        private KMeans _kMeans;
        public Form1()
        {
            InitializeComponent();
            _kmeansPointList = new BindingList<DPoint>();
            _dataPointList = new BindingList<Point>();
            gridData.DataSource = _dataPointList;
            gridData.Columns[0].Width = 50;
            gridData.Columns[1].Width = 50;

            _dataSeries = new Series("Data Points");
            _dataSeries.ChartType = SeriesChartType.Point;
            chart1.Series.Clear();
            chart1.Series.Add(_dataSeries);
            _dataSeries.XValueMember = "X";
            _dataSeries.YValueMembers = "Y";
            chart1.DataSource = _dataPointList;

            _kMeansSeries = new Series("KMeans");
            _kMeansSeries.ChartType = SeriesChartType.Line;
            chart2.Series.Clear();
            chart2.Series.Add(_kMeansSeries);
            _kMeansSeries.XValueMember = "X";
            _kMeansSeries.YValueMembers = "Y";
            chart2.DataSource = _kmeansPointList;
        }

        private void btnAdd_Click(object sender, System.EventArgs e)
        {
            _dataPointList.Add(new Point((int)X.Value, (int)Y.Value));
            chart1.DataBind();
        }

        private void button1_Click(object sender, System.EventArgs e)
        {
            List<double[]> trainingSetList = new List<double[]>();
            foreach(Point p in _dataPointList)
            {
                trainingSetList.Add(new double[2] { p.X, p.Y });
            }

            double[][] trainingSet = trainingSetList.ToArray();

            IDistance eucledeanDistance = new EuclideanDistance();

            BackgroundWorker backgroundWorker = new BackgroundWorker();
            backgroundWorker.WorkerReportsProgress = true;
            backgroundWorker.DoWork += (o, eventArgs) =>
            {
                for (int k = (int)start.Value; k <= (int)end.Value; k++)
                {
                    List<double[]> distanceResult = new List<double[]>();

                    _kMeans = new KMeans(k, eucledeanDistance);
                    Centroid[] centroids = _kMeans.Run(trainingSetList.ToArray());

                    double result = 0;
                    foreach (double[] point in trainingSetList)
                    {
                        double minValue = Double.MaxValue;
                        foreach (Centroid centroid in centroids)
                        {
                            double distance = eucledeanDistance.Run(point, centroid.Array);
                            if (distance < minValue)
                                minValue = distance;
                        }
                        result += minValue;
                    }
                    result /= trainingSetList.Count;
                    _kmeansPointList.Add(new DPoint(k, result));

                    double progressPercent = (100 * k) / (int)end.Value;
                    ((BackgroundWorker)o).ReportProgress((int)progressPercent, new UserState { Distortion = result, Index = k });
                }
            };
            backgroundWorker.ProgressChanged += (o, eventArgsProgressChanged) =>
            {
                chart2.DataBind();
                progressBar1.Value = eventArgsProgressChanged.ProgressPercentage;
                UserState temp = eventArgsProgressChanged.UserState as UserState;
                listBox1.Items.Add(String.Format("K={0}, Dist: {1}", temp.Index, Math.Round(temp.Distortion, 2)));
            };
            backgroundWorker.RunWorkerAsync();
        }

        private void btnLoadData_Click(object sender, EventArgs e)
        {
            _dataPointList.Add(new Point(3,5));
            _dataPointList.Add(new Point(1, 4));
            _dataPointList.Add(new Point(1, 5));
            _dataPointList.Add(new Point(2, 6));
            _dataPointList.Add(new Point(6,8));
            _dataPointList.Add(new Point(6,6));
            _dataPointList.Add(new Point(6,7));
            _dataPointList.Add(new Point(5,6));
            _dataPointList.Add(new Point(7,1));
            _dataPointList.Add(new Point(8,2));
            _dataPointList.Add(new Point(9,1));
            _dataPointList.Add(new Point(9,3));
            _dataPointList.Add(new Point(9,2));
            _dataPointList.Add(new Point(8,3));
            chart1.DataBind();
        }
    }
}
